"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const default_fetcher_1 = require("@pnpm/default-fetcher");
const default_resolver_1 = require("@pnpm/default-resolver");
const logger_1 = require("@pnpm/logger");
const server_1 = require("@pnpm/server");
const fs = require("graceful-fs");
const mkdirp = require("mkdirp-promise");
const package_store_1 = require("package-store");
const path = require("path");
const onExit = require("signal-exit");
const extendOptions_1 = require("supi/lib/api/extendOptions");
const writeJsonFile = require("write-json-file");
exports.default = (input, opts) => __awaiter(this, void 0, void 0, function* () {
    logger_1.default.warn('The store server is an experimental feature. Breaking changes may happen in non-major versions.');
    const strictOpts = yield extendOptions_1.default(opts);
    const resolve = default_resolver_1.default(strictOpts);
    const fetchers = default_fetcher_1.default(strictOpts);
    const storeCtrl = yield package_store_1.default(resolve, fetchers, {
        lockStaleDuration: strictOpts.lockStaleDuration,
        locks: strictOpts.locks,
        networkConcurrency: strictOpts.networkConcurrency,
        store: strictOpts.store,
    });
    // the store folder will be needed because server will want to create a file there
    // for the IPC connection
    yield mkdirp(strictOpts.store);
    const ipcConnectionPath = createIpcConnectionPath(strictOpts.store);
    const connectionOptions = {
        remotePrefix: `http://unix:${ipcConnectionPath}:`,
    };
    const serverJsonPath = path.join(strictOpts.store, 'server.json');
    yield writeJsonFile(serverJsonPath, { connectionOptions });
    const serverOptions = {
        path: ipcConnectionPath,
    };
    const server = server_1.createServer(storeCtrl, serverOptions);
    onExit(() => {
        server.close();
        fs.unlinkSync(serverJsonPath);
    });
});
function createIpcConnectionPath(fsPath) {
    fsPath = path.normalize(fsPath) + path.sep + 'socket';
    if (process.platform === 'win32') {
        return `\\\\.\\pipe\\${fsPath}`;
    }
    return fsPath;
}
//# sourceMappingURL=server.js.map